@@ -1,47 +1,28 @@
 /*
- * Copyright (c) 1983, 1995-1997 Eric P. Allman
+ * Copyright (c) 1998 Sendmail, Inc.  All rights reserved.
+ * Copyright (c) 1983, 1995-1997 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
  *	The Regents of the University of California.  All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * By using this file, you agree to the terms and conditions set
+ * forth in the LICENSE file which can be found at the top level of
+ * the sendmail distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)deliver.c	8.285 (Berkeley) 8/2/97";
+static char sccsid[] = "@(#)deliver.c	8.349 (Berkeley) 5/19/98";
 #endif /* not lint */
 
 #include "sendmail.h"
 #include <errno.h>
+#include <grp.h>
 #if NAMED_BIND
 #include <resolv.h>
+#endif
 
-extern int	h_errno;
+#if HASSETUSERCONTEXT
+# include <login_cap.h>
 #endif
 
 #if SMTP
@@ -77,9 +58,25 @@ sendall(e, mode)
 	register ENVELOPE *ee;
 	ENVELOPE *splitenv = NULL;
 	int oldverbose = Verbose;
-	bool somedeliveries = FALSE;
+	bool somedeliveries = FALSE, expensive = FALSE;
 	pid_t pid;
-	extern void sendenvelope();
+	void sendenvelope __P((ENVELOPE *, int));
+
+	/*
+	**  If this message is to be discarded, don't bother sending
+	**  the message at all.
+	*/
+
+	if (bitset(EF_DISCARD, e->e_flags))
+	{
+		if (tTd(13, 1))
+			printf("sendall: discarding id %s\n", e->e_id);
+		e->e_flags |= EF_CLRQUEUE;
+		if (LogLevel > 4)
+			sm_syslog(LOG_INFO, e->e_id, "discarded");
+		markstats(e, NULL, TRUE);
+		return;
+	}
 
 	/*
 	**  If we have had global, fatal errors, don't bother sending
@@ -106,7 +103,7 @@ sendall(e, mode)
 
 	if (tTd(13, 1))
 	{
-		extern void printenvflags();
+		extern void printenvflags __P((ENVELOPE *));
 
 		printf("\n===== SENDALL: mode %c, id %s, e_from ",
 			mode, e->e_id);
@@ -281,6 +278,7 @@ sendall(e, mode)
 				if (tTd(13, 30))
 					printf("    ... expensive\n");
 				q->q_flags |= QQUEUEUP;
+				expensive = TRUE;
 			}
 			else
 			{
@@ -292,8 +290,8 @@ sendall(e, mode)
 
 		if (owner != NULL && otherowners > 0)
 		{
-			extern HDR *copyheader();
-			extern ADDRESS *copyqueue();
+			extern HDR *copyheader __P((HDR *));
+			extern ADDRESS *copyqueue __P((ADDRESS *));
 			extern void dup_queue_file __P((ENVELOPE *, ENVELOPE *, int));
 
 			/*
@@ -393,11 +391,13 @@ sendall(e, mode)
 
 		/* treat this as a delivery in terms of counting tries */
 		e->e_dtime = curtime();
-		e->e_ntries++;
+		if (!expensive)
+			e->e_ntries++;
 		for (ee = splitenv; ee != NULL; ee = ee->e_sibling)
 		{
 			ee->e_dtime = curtime();
-			ee->e_ntries++;
+			if (!expensive)
+				ee->e_ntries++;
 		}
 	}
 
@@ -446,10 +446,12 @@ sendall(e, mode)
 
 	  case SM_QUEUE:
 	  case SM_DEFER:
+# if HASFLOCK
   queueonly:
+# endif
 		if (e->e_nrcpts > 0)
 			e->e_flags |= EF_INQUEUE;
-		dropenvelope(e, FALSE);
+		dropenvelope(e, splitenv != NULL);
 		for (ee = splitenv; ee != NULL; ee = ee->e_sibling)
 		{
 			if (ee->e_nrcpts > 0)
@@ -476,7 +478,7 @@ sendall(e, mode)
 
 			/* now drop the envelope in the parent */
 			e->e_flags |= EF_INQUEUE;
-			dropenvelope(e, FALSE);
+			dropenvelope(e, splitenv != NULL);
 
 			/* arrange to reacquire lock after fork */
 			e->e_id = qid;
@@ -500,23 +502,30 @@ sendall(e, mode)
 		pid = fork();
 		if (pid < 0)
 		{
+# if HASFLOCK
 			goto queueonly;
+# else
+			e->e_id = NULL;
+			for (ee = splitenv; ee != NULL; ee = ee->e_sibling)
+				ee->e_id = NULL;
+			return;
+# endif /* HASFLOCK */
 		}
 		else if (pid > 0)
 		{
 # if HASFLOCK
 			/* be sure we leave the temp files to our child */
-			/* can't call unlockqueue to avoid unlink of xfp */
-			if (e->e_lockfp != NULL)
-				(void) xfclose(e->e_lockfp, "sendenvelope lockfp", e->e_id);
-			e->e_lockfp = NULL;
-
 			/* close any random open files in the envelope */
 			closexscript(e);
 			if (e->e_dfp != NULL)
 				(void) xfclose(e->e_dfp, "sendenvelope dfp", e->e_id);
 			e->e_dfp = NULL;
 			e->e_flags &= ~EF_HAS_DF;
+
+			/* can't call unlockqueue to avoid unlink of xfp */
+			if (e->e_lockfp != NULL)
+				(void) xfclose(e->e_lockfp, "sendenvelope lockfp", e->e_id);
+			e->e_lockfp = NULL;
 # endif
 
 			/* make sure the parent doesn't own the envelope */
@@ -538,7 +547,11 @@ sendall(e, mode)
 		/* prevent parent from waiting if there was an error */
 		if (pid < 0)
 		{
+# if HASFLOCK
 			e->e_flags |= EF_INQUEUE;
+# else
+			e->e_id = NULL;
+# endif /* HASFLOCK */
 			finis();
 		}
 
@@ -565,8 +578,13 @@ sendall(e, mode)
 		**  Now reacquire and run the various queue files.
 		*/
 
-		for (ee = splitenv; ee != NULL; ee = e->e_sibling)
+		for (ee = splitenv; ee != NULL; ee = ee->e_sibling)
+		{
+			ENVELOPE *sibling = ee->e_sibling;
+
 			(void) dowork(ee->e_id, FALSE, FALSE, ee);
+			ee->e_sibling = sibling;
+		}
 		(void) dowork(e->e_id, FALSE, FALSE, e);
 		finis();
 # endif /* !HASFLOCK */
@@ -592,7 +610,7 @@ sendall(e, mode)
 void
 sendenvelope(e, mode)
 	register ENVELOPE *e;
-	char mode;
+	int mode;
 {
 	register ADDRESS *q;
 	bool didany;
@@ -844,10 +862,11 @@ deliver(e, firstto)
 	volatile bool clever = FALSE;	/* running user smtp to this mailer */
 	ADDRESS *volatile tochain = NULL; /* users chain in this mailer call */
 	int rcode;			/* response code */
+	int lmtp_rcode = EX_OK;
 	char *firstsig;			/* signature of firstto */
 	pid_t pid = -1;
 	char *volatile curhost;
-	register volatile u_short port = 0;
+	register u_short port = 0;
 	time_t xstart;
 	bool suidwarn;
 	bool anyok;			/* at least one address was OK */
@@ -858,7 +877,7 @@ deliver(e, firstto)
 	char tobuf[TOBUFSIZE];		/* text line of to people */
 	char buf[MAXNAME + 1];
 	char rpathbuf[MAXNAME + 1];	/* translated return path */
-	extern int checkcompat();
+	extern int checkcompat __P((ADDRESS *, ENVELOPE *));
 	extern void markfailure __P((ENVELOPE *, ADDRESS *, MCI *, int));
 
 	errno = 0;
@@ -891,6 +910,18 @@ deliver(e, firstto)
 	if (tTd(10, 100))
 		printopenfds(FALSE);
 
+	/*
+	**  Clear $&{client_*} macros if this is a bounce message to
+	**  prevent rejection by check_compat ruleset.
+	*/
+	
+	if (bitset(EF_RESPONSE, e->e_flags))
+	{
+		define(macid("{client_name}", NULL), "", e);
+		define(macid("{client_addr}", NULL), "", e);
+		define(macid("{client_port}", NULL), "", e);
+	}
+	
 	/*
 	**  Do initial argv setup.
 	**	Insert the mailer name.  Notice that $x expansion is
@@ -1104,13 +1135,26 @@ deliver(e, firstto)
 
 		if (strcmp(m->m_mailer, "[FILE]") == 0)
 		{
-			rcode = mailfile(user, ctladdr, SFF_CREAT, e);
+			define('u', user, e);	/* to user */
+			p = to->q_home;
+			if (p == NULL && ctladdr != NULL)
+				p = ctladdr->q_home;
+			define('z', p, e);	/* user's home */
+			expand(m->m_argv[1], buf, sizeof buf, e);
+			if (strlen(buf) > 0)
+				rcode = mailfile(buf, m, ctladdr, SFF_CREAT, e);
+			else
+			{
+				syserr("empty filename specification for mailer %s",
+				       m->m_name);
+				rcode = EX_CONFIG;
+			}
 			giveresponse(rcode, m, NULL, ctladdr, xstart, e);
+			markfailure(e, to, NULL, rcode);
 			e->e_nsent++;
 			if (rcode == EX_OK)
 			{
 				to->q_flags |= QSENT;
-				markstats(e, to);
 				if (bitnset(M_LOCALMAILER, m->m_flags) &&
 				    bitset(QPINGONSUCCESS, to->q_flags))
 				{
@@ -1121,6 +1165,7 @@ deliver(e, firstto)
 				}
 			}
 			to->q_statdate = curtime();
+			markstats(e, to, FALSE);
 			continue;
 		}
 
@@ -1307,7 +1352,6 @@ deliver(e, firstto)
 tryhost:
 		while (*curhost != '\0')
 		{
-			register char *p;
 			static char hostbuf[MAXNAME + 1];
 			extern int makeconnection __P((char *, u_short, MCI *, ENVELOPE *));
 
@@ -1368,7 +1412,7 @@ tryhost:
 					hostbuf, m->m_name);
 			else
 				message("Connecting to %s port %d via %s...",
-					hostbuf, port, m->m_name);
+					hostbuf, ntohs(port), m->m_name);
 			i = makeconnection(hostbuf, port, mci, e);
 			mci->mci_lastuse = curtime();
 			mci->mci_exitstat = i;
@@ -1420,6 +1464,24 @@ tryhost:
 	{
 		/* flush any expired connections */
 		(void) mci_scan(NULL);
+		mci = NULL;
+
+#if SMTP
+		if (bitnset(M_LMTP, m->m_flags))
+		{
+			/* try to get a cached connection */
+			mci = mci_get(m->m_name, m);
+			if (mci->mci_host == NULL)
+				mci->mci_host = m->m_name;
+			CurHostName = mci->mci_host;
+			if (mci->mci_state != MCIS_CLOSED)
+			{
+				message("Using cached LMTP connection for %s...",
+					m->m_name);
+				goto do_transfer;
+			}
+		}
+#endif
 
 		/* announce the connection to verbose listeners */
 		if (host == NULL || host[0] == '\0')
@@ -1517,9 +1579,7 @@ tryhost:
 		if (e->e_xfp != NULL)
 			(void) fflush(e->e_xfp);		/* for debugging */
 		(void) fflush(stdout);
-# ifdef SIGCHLD
 		(void) setsignal(SIGCHLD, SIG_DFL);
-# endif /* SIGCHLD */
 		DOFORK(FORK);
 		/* pid is set by DOFORK */
 		if (pid < 0)
@@ -1577,7 +1637,7 @@ tryhost:
 					pwd = sm_getpwnam(contextaddr->q_user);
 				if (pwd != NULL)
 					(void) setusercontext(NULL,
-						pwd, pwd->m_uid,
+						pwd, pwd->pw_uid,
 						LOGIN_SETRESOURCES|LOGIN_SETPRIORITY);
 			}
 #endif
@@ -1594,15 +1654,42 @@ tryhost:
 			else if (ctladdr != NULL && ctladdr->q_gid != 0)
 			{
 				if (!DontInitGroups)
-					(void) initgroups(ctladdr->q_ruser != NULL ?
-						ctladdr->q_ruser : ctladdr->q_user,
-						ctladdr->q_gid);
+				{
+					char *u = ctladdr->q_ruser;
+
+					if (u == NULL)
+						u = ctladdr->q_user;
+
+					if (initgroups(u, ctladdr->q_gid) == -1 && suidwarn)
+						syserr("openmailer: initgroups(%s, %d) failed",
+							u, ctladdr->q_gid);
+				}
+				else
+				{
+					GIDSET_T gidset[1];
+
+					gidset[0] = ctladdr->q_gid;
+					if (setgroups(1, gidset) == -1 && suidwarn)
+						syserr("openmailer: setgroups() failed");
+				}
 				new_gid = ctladdr->q_gid;
 			}
 			else
 			{
 				if (!DontInitGroups)
-					(void) initgroups(DefUser, DefGid);
+				{
+					if (initgroups(DefUser, DefGid) == -1 && suidwarn)
+						syserr("openmailer: initgroups(%s, %d) failed",
+							DefUser, DefGid);
+				}
+				else
+				{
+					GIDSET_T gidset[1];
+
+					gidset[0] = DefGid;
+					if (setgroups(1, gidset) == -1 && suidwarn)
+						syserr("openmailer: setgroups() failed");
+				}
 				if (m->m_gid == 0)
 					new_gid = DefGid;
 				else
@@ -1616,13 +1703,13 @@ tryhost:
 			endpwent();
 			if (bitnset(M_SPECIFIC_UID, m->m_flags))
 				new_euid = m->m_uid;
-			if (bitset(S_ISUID, stb.st_mode))
+			else if (bitset(S_ISUID, stb.st_mode))
 				new_ruid = stb.st_uid;
 			else if (ctladdr != NULL && ctladdr->q_uid != 0)
 				new_ruid = ctladdr->q_uid;
 			else if (m->m_uid != 0)
 				new_ruid = m->m_uid;
-			else if (!bitnset(M_SPECIFIC_UID, m->m_flags))
+			else
 				new_ruid = DefUid;
 			if (new_euid != NO_UID)
 			{
@@ -1658,7 +1745,7 @@ tryhost:
 			/* move into some "safe" directory */
 			if (m->m_execdir != NULL)
 			{
-				char *p, *q;
+				char *q;
 				char buf[MAXLINE + 1];
 
 				for (p = m->m_execdir; p != NULL; p = q)
@@ -1691,8 +1778,7 @@ tryhost:
 				}
 				(void) close(rpvect[1]);
 			}
-			else if (OpMode == MD_SMTP || OpMode == MD_DAEMON ||
-				  HoldErrs || DisConnected)
+			else
 			{
 				/* put mailer output in transcript */
 				if (dup2(fileno(e->e_xfp), STDOUT_FILENO) < 0)
@@ -1748,10 +1834,21 @@ tryhost:
 		**  Set up return value.
 		*/
 
-		mci = (MCI *) xalloc(sizeof *mci);
-		bzero((char *) mci, sizeof *mci);
+		if (mci == NULL)
+		{
+			mci = (MCI *) xalloc(sizeof *mci);
+			bzero((char *) mci, sizeof *mci);
+		}
 		mci->mci_mailer = m;
-		mci->mci_state = clever ? MCIS_OPENING : MCIS_OPEN;
+		if (clever)
+		{
+			mci->mci_state = MCIS_OPENING;
+			mci_cache(mci);
+		}
+		else
+		{
+			mci->mci_state = MCIS_OPEN;
+		}
 		mci->mci_pid = pid;
 		(void) close(mpvect[0]);
 		mci->mci_out = fdopen(mpvect[1], "w");
@@ -1810,6 +1907,7 @@ tryhost:
 	}
 #endif
 
+do_transfer:
 	/* clear out per-message flags from connection structure */
 	mci->mci_flags &= ~(MCIF_CVT7TO8|MCIF_CVT8TO7);
 
@@ -1871,6 +1969,7 @@ tryhost:
 		**  Format and send message.
 		*/
 
+		mci->mci_contentlen = 0;
 		putfromline(mci, e);
 		(*e->e_puthdr)(mci, e->e_header, e);
 		(*e->e_putbody)(mci, e, NULL);
@@ -1884,7 +1983,6 @@ tryhost:
 		extern int smtpmailfrom __P((MAILER *, MCI *, ENVELOPE *));
 		extern int smtprcpt __P((ADDRESS *, MAILER *, MCI *, ENVELOPE *));
 		extern int smtpdata __P((MAILER *, MCI *, ENVELOPE *));
-		extern int smtpgetstat __P((MAILER *, MCI *, ENVELOPE *));
 
 		/*
 		**  Send the MAIL FROM: protocol
@@ -1901,7 +1999,7 @@ tryhost:
 			for (to = tochain; to != NULL; to = to->q_tchain)
 			{
 				e->e_to = to->q_paddr;
-				if (strlen(to->q_paddr) + (t - tobuf) + 2 >= sizeof tobuf)
+				if (strlen(to->q_paddr) + (t - tobuf) + 2 > sizeof tobuf)
 				{
 					/* not enough room */
 					continue;
@@ -1966,14 +2064,13 @@ tryhost:
 
   give_up:
 #if SMTP
-# if _FFR_LMTP
 	if (bitnset(M_LMTP, m->m_flags))
 	{
+		lmtp_rcode = rcode;
 		tobuf[0] = '\0';
 		anyok = FALSE;
 	}
 	else
-# endif
 #endif
 		anyok = rcode == EX_OK;
 
@@ -1984,11 +2081,13 @@ tryhost:
 			continue;
 
 #if SMTP
-# if _FFR_LMTP
 		/* if running LMTP, get the status for each address */
 		if (bitnset(M_LMTP, m->m_flags))
 		{
-			rcode = smtpgetstat(m, mci, e);
+			extern int smtpgetstat __P((MAILER *, MCI *, ENVELOPE *));
+
+			if (lmtp_rcode == EX_OK)
+				rcode = smtpgetstat(m, mci, e);
 			if (rcode == EX_OK)
 			{
 				if (strlen(to->q_paddr) + strlen(tobuf) + 2 >= sizeof tobuf)
@@ -2012,7 +2111,6 @@ tryhost:
 			}
 		}
 		else
-# endif
 #endif
 		{
 			/* mark bad addresses */
@@ -2048,7 +2146,6 @@ tryhost:
 	}
 
 #if SMTP
-# if _FFR_LMTP
 	if (bitnset(M_LMTP, m->m_flags))
 	{
 		/*
@@ -2063,13 +2160,12 @@ tryhost:
 		if (mci != NULL && mci->mci_state == MCIS_ACTIVE)
 			mci->mci_state = MCIS_OPEN;
 	}
-# endif
 #endif
 
 	if (tobuf[0] != '\0')
 		giveresponse(rcode, m, mci, ctladdr, xstart, e);
 	if (anyok)
-		markstats(e, tochain);
+		markstats(e, tochain, FALSE);
 	mci_store_persistent(mci);
 
 #if SMTP
@@ -2148,7 +2244,10 @@ markfailure(e, q, mci, rcode)
 	if (mci != NULL && mci->mci_status != NULL)
 	{
 		q->q_status = mci->mci_status;
-		q->q_rstatus = mci->mci_rstatus;
+		if (mci->mci_rstatus != NULL)
+			q->q_rstatus = newstr(mci->mci_rstatus);
+		else
+			q->q_rstatus = NULL;
 	}
 	else if (e->e_status != NULL)
 	{
@@ -2244,6 +2343,8 @@ endmailer(mci, e, pv)
 {
 	int st;
 
+	mci_unlock_host(mci);
+
 	/* close any connections */
 	if (mci->mci_in != NULL)
 		(void) xfclose(mci->mci_in, mci->mci_mailer->m_name, "mci_in");
@@ -2331,6 +2432,9 @@ giveresponse(stat, m, mci, ctladdr, xstart, e)
 	extern int N_SysEx;
 	char buf[MAXLINE];
 
+	if (e == NULL)
+		syserr("giveresponse: null envelope");
+
 	/*
 	**  Compute status message from code.
 	*/
@@ -2346,7 +2450,7 @@ giveresponse(stat, m, mci, ctladdr, xstart, e)
 			statmsg = buf;
 		}
 	}
-	else if (i < 0 || i > N_SysEx)
+	else if (i < 0 || i >= N_SysEx)
 	{
 		(void) snprintf(buf, sizeof buf, "554 unknown mailer error %d",
 			stat);
@@ -2392,7 +2496,7 @@ giveresponse(stat, m, mci, ctladdr, xstart, e)
 	else
 	{
 		statmsg = SysExMsg[i];
-		if (*statmsg++ == ':')
+		if (*statmsg++ == ':' && errno != 0)
 		{
 			(void) snprintf(buf, sizeof buf, "%s: %s",
 				statmsg, errstring(errno));
@@ -2428,7 +2532,8 @@ giveresponse(stat, m, mci, ctladdr, xstart, e)
 	**	that.
 	*/
 
-	if (LogLevel > ((stat == EX_TEMPFAIL) ? 8 : (stat == EX_OK) ? 7 : 6))
+	if (OpMode != MD_VERIFY && !bitset(EF_VRFYONLY, e->e_flags) &&
+	    LogLevel > ((stat == EX_TEMPFAIL) ? 8 : (stat == EX_OK) ? 7 : 6))
 		logdelivery(m, mci, &statmsg[4], ctladdr, xstart, e);
 
 	if (tTd(11, 2))
@@ -2542,8 +2647,7 @@ logdelivery(m, mci, stat, ctladdr, xstart, e)
 	}
 	else if (strcmp(stat, "queued") != 0)
 	{
-		char *p = macvalue('h', e);
-
+		p = macvalue('h', e);
 		if (p != NULL && p[0] != '\0')
 		{
 			snprintf(bp, SPACELEFT(buf, bp), ", relay=%s",
@@ -2660,8 +2764,7 @@ logdelivery(m, mci, stat, ctladdr, xstart, e)
 	}
 	else if (strcmp(stat, "queued") != 0)
 	{
-		char *p = macvalue('h', e);
-
+		p = macvalue('h', e);
 		if (p != NULL && p[0] != '\0')
 			snprintf(buf, sizeof buf, "relay=%.100s", p);
 	}
@@ -2704,6 +2807,8 @@ putfromline(mci, e)
 	if (bitnset(M_NHDR, mci->mci_mailer->m_flags))
 		return;
 
+	mci->mci_flags |= MCIF_INHEADER;
+
 	if (bitnset(M_UGLYUUCP, mci->mci_mailer->m_flags))
 	{
 		char *bang;
@@ -2741,7 +2846,7 @@ putfromline(mci, e)
 		}
 	}
 	expand(template, buf, sizeof buf, e);
-	putxline(buf, strlen(buf), mci, PXLF_NOTHINGSPECIAL);
+	putxline(buf, strlen(buf), mci, PXLF_HEADER);
 }
 /*
 **  PUTBODY -- put the body of a message.
@@ -2832,6 +2937,7 @@ putbody(mci, e, separator)
 
 		/* now do the hard work */
 		boundaries[0] = NULL;
+		mci->mci_flags |= MCIF_INHEADER;
 		mime8to7(mci, e->e_header, e, boundaries, M87F_OUTER);
 	}
 # if MIME7TO8
@@ -2851,6 +2957,7 @@ putbody(mci, e, separator)
 		int padc;
 		char *buflim;
 		int pos = 0;
+		size_t eol_len;
 		char peekbuf[10];
 
 		/* we can pass it through unmodified */
@@ -2865,6 +2972,7 @@ putbody(mci, e, separator)
 		if (mci->mci_mailer->m_linelimit > 0 &&
 		    mci->mci_mailer->m_linelimit < sizeof buf - 1)
 			buflim = &buf[mci->mci_mailer->m_linelimit - 1];
+		eol_len = strlen(mci->mci_mailer->m_eol);
 
 		/* copy temp file to output with mapping */
 		ostate = OS_HEAD;
@@ -2933,14 +3041,19 @@ putbody(mci, e, separator)
 				if (padc != EOF)
 				{
 					putc(padc, mci->mci_out);
+					mci->mci_contentlen++;
 					pos++;
 				}
 				for (xp = buf; xp < bp; xp++)
+				{
 					putc(*xp, mci->mci_out);
+					mci->mci_contentlen++;
+				}
 				if (c == '\n')
 				{
 					fputs(mci->mci_mailer->m_eol,
 					      mci->mci_out);
+					mci->mci_contentlen += eol_len;
 					pos = 0;
 				}
 				else
@@ -2965,6 +3078,7 @@ putbody(mci, e, separator)
 				{
 					/* got CRLF */
 					fputs(mci->mci_mailer->m_eol, mci->mci_out);
+					mci->mci_contentlen += eol_len;
 					if (TrafficLogFile != NULL)
 					{
 						fputs(mci->mci_mailer->m_eol,
@@ -2997,7 +3111,9 @@ putch:
 				    c != '\n')
 				{
 					putc('!', mci->mci_out);
+					mci->mci_contentlen++;
 					fputs(mci->mci_mailer->m_eol, mci->mci_out);
+					mci->mci_contentlen += eol_len;
 					if (TrafficLogFile != NULL)
 					{
 						fprintf(TrafficLogFile, "!%s",
@@ -3013,6 +3129,7 @@ putch:
 						fputs(mci->mci_mailer->m_eol,
 						      TrafficLogFile);
 					fputs(mci->mci_mailer->m_eol, mci->mci_out);
+					mci->mci_contentlen += eol_len;
 					pos = 0;
 					ostate = OS_HEAD;
 				}
@@ -3021,6 +3138,7 @@ putch:
 					if (TrafficLogFile != NULL)
 						putc(c, TrafficLogFile);
 					putc(c, mci->mci_out);
+					mci->mci_contentlen++;
 					pos++;
 					ostate = OS_INLINE;
 				}
@@ -3037,7 +3155,10 @@ putch:
 					putc(*xp, TrafficLogFile);
 			}
 			for (xp = buf; xp < bp; xp++)
+			{
 				putc(*xp, mci->mci_out);
+				mci->mci_contentlen++;
+			}
 			pos += bp - buf;
 		}
 		if (pos > 0)
@@ -3045,6 +3166,7 @@ putch:
 			if (TrafficLogFile != NULL)
 				fputs(mci->mci_mailer->m_eol, TrafficLogFile);
 			fputs(mci->mci_mailer->m_eol, mci->mci_out);
+			mci->mci_contentlen += eol_len;
 		}
 	}
 
@@ -3084,6 +3206,8 @@ endofmessage:
 **
 **	Parameters:
 **		filename -- the name of the file to send to.
+**		mailer -- mailer definition for recipient -- if NULL,
+**			use FileMailer.
 **		ctladdr -- the controlling address header -- includes
 **			the userid/groupid to be when sending.
 **		sfflags -- flags for opening.
@@ -3096,17 +3220,23 @@ endofmessage:
 **		none.
 */
 
+static jmp_buf	CtxMailfileTimeout;
+static void	mailfiletimeout __P((void));
+
 int
-mailfile(filename, ctladdr, sfflags, e)
-	char *filename;
+mailfile(filename, mailer, ctladdr, sfflags, e)
+	char *volatile filename;
+	MAILER *volatile mailer;
 	ADDRESS *ctladdr;
-	int sfflags;
+	volatile int sfflags;
 	register ENVELOPE *e;
 {
 	register FILE *f;
 	register pid_t pid = -1;
-	int mode = ST_MODE_NOFILE;
+	volatile int mode = ST_MODE_NOFILE;
 	bool suidwarn = geteuid() == 0;
+	char *p;
+	EVENT *ev;
 
 	if (tTd(11, 1))
 	{
@@ -3114,6 +3244,9 @@ mailfile(filename, ctladdr, sfflags, e)
 		printaddr(ctladdr, FALSE);
 	}
 
+	if (mailer == NULL)
+		mailer = FileMailer;
+
 	if (e->e_xfp != NULL)
 		fflush(e->e_xfp);
 
@@ -3125,6 +3258,19 @@ mailfile(filename, ctladdr, sfflags, e)
 	if (strcmp(filename, "/dev/null") == 0)
 		return EX_OK;
 
+	/* check for 8-bit available */
+	if (bitset(EF_HAS8BIT, e->e_flags) &&
+	    bitnset(M_7BITS, mailer->m_flags) &&
+	    (bitset(EF_DONT_MIME, e->e_flags) ||
+	     !(bitset(MM_MIME8BIT, MimeMode) ||
+	       (bitset(EF_IS_MIME, e->e_flags) &&
+		bitset(MM_CVTMIME, MimeMode)))))
+	{
+		usrerr("554 Cannot send 8-bit data to 7-bit destination");
+		e->e_status = "5.6.3";
+		return(EX_DATAERR);
+	}
+
 	/*
 	**  Fork so we can change permissions here.
 	**	Note that we MUST use fork, not vfork, because of
@@ -3140,7 +3286,7 @@ mailfile(filename, ctladdr, sfflags, e)
 		/* child -- actually write to file */
 		struct stat stb;
 		MCI mcibuf;
-		int oflags = O_WRONLY|O_APPEND;
+		volatile int oflags = O_WRONLY|O_APPEND;
 
 		if (e->e_lockfp != NULL)
 			(void) close(fileno(e->e_lockfp));
@@ -3152,6 +3298,16 @@ mailfile(filename, ctladdr, sfflags, e)
 		e->e_to = filename;
 		ExitStat = EX_OK;
 
+		if (setjmp(CtxMailfileTimeout) != 0)
+		{
+			exit(EX_TEMPFAIL);
+		}
+
+		if (TimeOuts.to_fileopen > 0)
+			ev = setevent(TimeOuts.to_fileopen, mailfiletimeout, 0);
+		else
+			ev = NULL;
+
 #ifdef HASLSTAT
 		if (lstat(filename, &stb) < 0)
 #else
@@ -3162,7 +3318,9 @@ mailfile(filename, ctladdr, sfflags, e)
 			mode = FileMode;
 			oflags |= O_CREAT|O_EXCL;
 		}
-		else if (bitset(0111, stb.st_mode) || stb.st_nlink != 1 ||
+		else if (bitset(S_IXUSR|S_IXGRP|S_IXOTH, stb.st_mode) ||
+			 (!bitset(DBS_FILEDELIVERYTOHARDLINK, DontBlameSendmail) &&
+			  stb.st_nlink != 1) ||
 			 (SafeFileEnv != NULL && !S_ISREG(stb.st_mode)))
 			exit(EX_CANTCREAT);
 		if (mode == ST_MODE_NOFILE)
@@ -3194,7 +3352,12 @@ mailfile(filename, ctladdr, sfflags, e)
 		/* select a new user to run as */
 		if (!bitset(SFF_RUNASREALUID, sfflags))
 		{
-			if (bitset(S_ISUID, mode))
+			if (bitnset(M_SPECIFIC_UID, mailer->m_flags))
+			{
+				RealUserName = NULL;
+				RealUid = mailer->m_uid;
+			}
+			else if (bitset(S_ISUID, mode))
 			{
 				RealUserName = NULL;
 				RealUid = stb.st_uid;
@@ -3207,10 +3370,10 @@ mailfile(filename, ctladdr, sfflags, e)
 					RealUserName = ctladdr->q_user;
 				RealUid = ctladdr->q_uid;
 			}
-			else if (FileMailer != NULL && FileMailer->m_uid != 0)
+			else if (mailer != NULL && mailer->m_uid != 0)
 			{
 				RealUserName = DefUser;
-				RealUid = FileMailer->m_uid;
+				RealUid = mailer->m_uid;
 			}
 			else
 			{
@@ -3219,12 +3382,14 @@ mailfile(filename, ctladdr, sfflags, e)
 			}
 
 			/* select a new group to run as */
-			if (bitset(S_ISGID, mode))
+			if (bitnset(M_SPECIFIC_UID, mailer->m_flags))
+				RealGid = mailer->m_gid;
+			else if (bitset(S_ISGID, mode))
 				RealGid = stb.st_gid;
 			else if (ctladdr != NULL && ctladdr->q_uid != 0)
 				RealGid = ctladdr->q_gid;
-			else if (FileMailer != NULL && FileMailer->m_gid != 0)
-				RealGid = FileMailer->m_gid;
+			else if (mailer != NULL && mailer->m_gid != 0)
+				RealGid = mailer->m_gid;
 			else
 				RealGid = DefGid;
 		}
@@ -3240,7 +3405,19 @@ mailfile(filename, ctladdr, sfflags, e)
 
 		/* set group id list (needs /etc/group access) */
 		if (RealUserName != NULL && !DontInitGroups)
-			(void) initgroups(RealUserName, RealGid);
+		{
+			if (initgroups(RealUserName, RealGid) == -1)
+				syserr("mailfile: initgroups(%s, %d) failed",
+					RealUserName, RealGid);
+		}
+		else
+		{
+			GIDSET_T gidset[1];
+
+			gidset[0] = RealGid;
+			if (setgroups(1, gidset) == -1)
+				syserr("mailfile: setgroups() failed");
+		}
 
 		/* if you have a safe environment, go into it */
 		if (SafeFileEnv != NULL && SafeFileEnv[0] != '\0')
@@ -3268,12 +3445,39 @@ mailfile(filename, ctladdr, sfflags, e)
 		if (setuid(RealUid) < 0 && suidwarn)
 			syserr("mailfile: setuid(%ld) failed", (long) RealUid);
 
-		sfflags |= SFF_NOPATHCHECK|SFF_NOLINK;
+		/* move into some "safe" directory */
+		if (mailer->m_execdir != NULL)
+		{
+			char *q;
+			char buf[MAXLINE + 1];
+
+			for (p = mailer->m_execdir; p != NULL; p = q)
+			{
+				q = strchr(p, ':');
+				if (q != NULL)
+					*q = '\0';
+				expand(p, buf, sizeof buf, e);
+				if (q != NULL)
+					*q++ = ':';
+				if (tTd(11, 20))
+					printf("mailfile: trydir %s\n",
+					       buf);
+				if (buf[0] != '\0' && chdir(buf) >= 0)
+					break;
+			}
+		}
+
+		sfflags |= SFF_NOPATHCHECK;
+		if (!bitset(DBS_FILEDELIVERYTOSYMLINK, DontBlameSendmail))
+			sfflags |= SFF_NOSLINK;
+		if (!bitset(DBS_FILEDELIVERYTOHARDLINK, DontBlameSendmail))
+			sfflags |= SFF_NOHLINK;
 		sfflags &= ~SFF_OPENASROOT;
 		f = safefopen(filename, oflags, FileMode, sfflags);
 		if (f == NULL)
 		{
-			message("554 cannot open: %s", errstring(errno));
+			message("554 cannot open %s: %s",
+				shortenstring(filename, 203), errstring(errno));
 			exit(EX_CANTCREAT);
 		}
 		if (filechanged(filename, fileno(f), &stb, sfflags))
@@ -3287,12 +3491,40 @@ mailfile(filename, ctladdr, sfflags, e)
 			exit(EX_CANTCREAT);
 		}
 
+		if (ev != NULL)
+			clrevent(ev);
+
 		bzero(&mcibuf, sizeof mcibuf);
-		mcibuf.mci_mailer = FileMailer;
+		mcibuf.mci_mailer = mailer;
 		mcibuf.mci_out = f;
-		if (bitnset(M_7BITS, FileMailer->m_flags))
+		mcibuf.mci_contentlen = 0;
+		if (bitnset(M_7BITS, mailer->m_flags))
 			mcibuf.mci_flags |= MCIF_7BIT;
 
+		/* clear out per-message flags from connection structure */
+		mcibuf.mci_flags &= ~(MCIF_CVT7TO8|MCIF_CVT8TO7);
+
+		if (bitset(EF_HAS8BIT, e->e_flags) &&
+		    !bitset(EF_DONT_MIME, e->e_flags) &&
+		    bitnset(M_7BITS, mailer->m_flags))
+			mcibuf.mci_flags |= MCIF_CVT8TO7;
+
+#if MIME7TO8
+		if (bitnset(M_MAKE8BIT, mailer->m_flags) &&
+		    !bitset(MCIF_7BIT, mcibuf.mci_flags) &&
+		    (p = hvalue("Content-Transfer-Encoding", e->e_header)) != NULL &&
+		    (strcasecmp(p, "quoted-printable") == 0 ||
+		     strcasecmp(p, "base64") == 0) &&
+		    (p = hvalue("Content-Type", e->e_header)) != NULL)
+		{
+			/* may want to convert 7 -> 8 */
+			/* XXX should really parse it here -- and use a class XXX */
+			if (strncasecmp(p, "text/plain", 10) == 0 &&
+			        (p[10] == '\0' || p[10] == ' ' || p[10] == ';'))
+				mcibuf.mci_flags |= MCIF_CVT7TO8;
+		}
+#endif
+
 		putfromline(&mcibuf, e);
 		(*e->e_puthdr)(&mcibuf, e->e_header, e);
 		(*e->e_putbody)(&mcibuf, e, NULL);
@@ -3321,17 +3553,29 @@ mailfile(filename, ctladdr, sfflags, e)
 		int st;
 
 		st = waitfor(pid);
+		if (st == -1)
+		{
+			syserr("mailfile: %s: wait", mailer->m_name);
+			return (EX_SOFTWARE);
+		}
 		if (WIFEXITED(st))
 			return (WEXITSTATUS(st));
 		else
 		{
-			syserr("child died on signal %d", st);
+			syserr("mailfile: %s: child died on signal %d",
+			       mailer->m_name, st);
 			return (EX_UNAVAILABLE);
 		}
 		/*NOTREACHED*/
 	}
 	return EX_UNAVAILABLE;	/* avoid compiler warning on IRIX */
 }
+
+static void
+mailfiletimeout()
+{
+	longjmp(CtxMailfileTimeout, 1);
+}
 /*
 **  HOSTSIGNATURE -- return the "signature" for a host.
 **
