@@ -1,44 +1,22 @@
 /*
- * Copyright (c) 1983, 1995-1997 Eric P. Allman
+ * Copyright (c) 1998 Sendmail, Inc.  All rights reserved.
+ * Copyright (c) 1983, 1995-1997 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
  *	The Regents of the University of California.  All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * By using this file, you agree to the terms and conditions set
+ * forth in the LICENSE file which can be found at the top level of
+ * the sendmail distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
  */
 
 # include "sendmail.h"
 
 #ifndef lint
 #if SMTP
-static char sccsid[] = "@(#)srvrsmtp.c	8.146 (Berkeley) 6/11/97 (with SMTP)";
+static char sccsid[] = "@(#)srvrsmtp.c	8.177 (Berkeley) 5/19/98 (with SMTP)";
 #else
-static char sccsid[] = "@(#)srvrsmtp.c	8.146 (Berkeley) 6/11/97 (without SMTP)";
+static char sccsid[] = "@(#)srvrsmtp.c	8.177 (Berkeley) 5/19/98 (without SMTP)";
 #endif
 #endif /* not lint */
 
@@ -50,7 +28,9 @@ static char sccsid[] = "@(#)srvrsmtp.c	8.146 (Berkeley) 6/11/97 (without SMTP)";
 **  SMTP -- run the SMTP protocol.
 **
 **	Parameters:
-**		none.
+**		nullserver -- if non-NULL, rejection message for
+**			all SMTP commands.
+**		e -- the envelope.
 **
 **	Returns:
 **		never.
@@ -118,7 +98,7 @@ static struct cmd	CmdTab[] =
 bool	OneXact = FALSE;		/* one xaction only this run */
 char	*CurSmtpClient;			/* who's at the other end of channel */
 
-static char	*skipword();
+static char	*skipword __P((char *volatile, char *));
 
 
 #define MAXBADCOMMANDS	25	/* maximum number of bad commands */
@@ -129,7 +109,7 @@ static char	*skipword();
 
 void
 smtp(nullserver, e)
-	bool nullserver;
+	char *nullserver;
 	register ENVELOPE *volatile e;
 {
 	register char *volatile p;
@@ -153,7 +133,9 @@ smtp(nullserver, e)
 	volatile int n_noop = 0;	/* count of NOOP/VERB/ONEX etc cmds */
 	volatile int n_helo = 0;	/* count of HELO/EHLO commands */
 	bool ok;
-	int lognullconnection = TRUE;
+	volatile int lognullconnection = TRUE;
+	register char *q;
+	QUEUE_CHAR *new;
 	char inp[MAXLINE];
 	char cmdbuf[MAXLINE];
 	extern ENVELOPE BlankEnvelope;
@@ -221,16 +203,7 @@ smtp(nullserver, e)
 	for (;;)
 	{
 		/* arrange for backout */
-		if (setjmp(TopFrame) > 0)
-		{
-			/* if() nesting is necessary for Cray UNICOS */
-			if (InChild)
-			{
-				QuickAbort = FALSE;
-				SuprErrs = TRUE;
-				finis();
-			}
-		}
+		(void) setjmp(TopFrame);
 		QuickAbort = FALSE;
 		HoldErrs = FALSE;
 		SuprErrs = FALSE;
@@ -260,6 +233,18 @@ smtp(nullserver, e)
 				sm_syslog(LOG_NOTICE, e->e_id,
 					"lost input channel from %.100s",
 					CurSmtpClient);
+			if (lognullconnection && LogLevel > 5)
+				sm_syslog(LOG_INFO, NULL,
+				"Null connection from %.100s",
+				CurSmtpClient);
+
+			/*
+			** If have not accepted mail (DATA), do not bounce
+			** bad addresses back to sender.
+			*/
+			if (bitset(EF_CLRQUEUE, e->e_flags))
+				e->e_sendqueue = NULL;
+
 			if (InChild)
 				ExitStat = EX_QUIT;
 			finis();
@@ -313,7 +298,7 @@ smtp(nullserver, e)
 		**	to everything.
 		*/
 
-		if (nullserver)
+		if (nullserver != NULL)
 		{
 			switch (c->cmdcode)
 			{
@@ -327,7 +312,7 @@ smtp(nullserver, e)
 			  default:
 				if (++badcommands > MAXBADCOMMANDS)
 					sleep(1);
-				usrerr("550 Access denied");
+				usrerr("550 %s", nullserver);
 				continue;
 			}
 		}
@@ -375,55 +360,62 @@ smtp(nullserver, e)
 					cmdbuf);
 				break;
 			}
-			else
-			{
-				register char *q;
 
-				for (q = p; *q != '\0'; q++)
-				{
-					if (!isascii(*q))
-						break;
-					if (isalnum(*q))
-						continue;
-					if (isspace(*q))
-					{
-						*q = '\0';
-						break;
-					}
-					if (strchr("[].-_#", *q) == NULL)
-						break;
-				}
-				if (*q != '\0')
-				{
-					if (!AllowBogusHELO)
-						usrerr("501 Invalid domain name");
-					else
-					{
-						message("250 %s Invalid domain name, accepting anyway",
-							MyHostName);
-						gothello = TRUE;
-					}
-					break;
-				}
-			}
-
-			sendinghost = newstr(p);
-			gothello = TRUE;
-			if (c->cmdcode != CMDEHLO)
+			/* check for long domain name (hides Received: info) */
+			if (strlen(p) > MAXNAME)
 			{
-				/* print old message and be done with it */
-				message("250 %s Hello %s, pleased to meet you",
-					MyHostName, CurSmtpClient);
+				usrerr("501 Invalid domain name");
 				break;
 			}
+
+			for (q = p; *q != '\0'; q++)
+			{
+				if (!isascii(*q))
+					break;
+				if (isalnum(*q))
+					continue;
+				if (isspace(*q))
+				{
+					*q = '\0';
+					break;
+				}
+				if (strchr("[].-_#", *q) == NULL)
+					break;
+			}
+			if (*q == '\0')
+			{
+				q = "pleased to meet you";
+				sendinghost = newstr(p);
+			}
+			else if (!AllowBogusHELO)
+			{
+				usrerr("501 Invalid domain name");
+				break;
+			}
+			else
+			{
+				q = "accepting invalid domain name";
+			}
+
+			gothello = TRUE;
 			
-			/* print extended message and brag */
-			message("250-%s Hello %s, pleased to meet you",
-				MyHostName, CurSmtpClient);
+			/* print HELO response message */
+			if (c->cmdcode != CMDEHLO || nullserver != NULL)
+			{
+				message("250 %s Hello %s, %s",
+					MyHostName, CurSmtpClient, q);
+				break;
+			}
+
+			message("250-%s Hello %s, %s",
+				MyHostName, CurSmtpClient, q);
+
+			/* print EHLO features list */
 			if (!bitset(PRIV_NOEXPN, PrivacyFlags))
 			{
 				message("250-EXPN");
-				message("250-VERB");
+				if (!bitset(PRIV_NOVERB, PrivacyFlags))
+					message("250-VERB");
 			}
 #if MIME8TO7
 			message("250-8BITMIME");
@@ -446,23 +438,14 @@ smtp(nullserver, e)
 			SmtpPhase = "server MAIL";
 
 			/* check for validity of this command */
-			if (!gothello)
+			if (!gothello && bitset(PRIV_NEEDMAILHELO, PrivacyFlags))
 			{
-				/* set sending host to our known value */
-				if (sendinghost == NULL)
-					sendinghost = peerhostname;
-
-				if (bitset(PRIV_NEEDMAILHELO, PrivacyFlags))
-				{
-					usrerr("503 Polite people say HELO first");
-					break;
-				}
+				usrerr("503 Polite people say HELO first");
+				break;
 			}
 			if (gotmail)
 			{
 				usrerr("503 Sender already specified");
-				if (InChild)
-					finis();
 				break;
 			}
 			if (InChild)
@@ -472,6 +455,10 @@ smtp(nullserver, e)
 				finis();
 			}
 
+			/* make sure we know who the sending host is */
+			if (sendinghost == NULL)
+				sendinghost = peerhostname;
+
 			p = skipword(p, "from");
 			if (p == NULL)
 				break;
@@ -479,6 +466,8 @@ smtp(nullserver, e)
 			/* fork a subprocess to process this command */
 			if (runinchild("SMTP-MAIL", e) > 0)
 				break;
+			if (Errors > 0)
+				goto undo_subproc_no_pm;
 			if (!gothello)
 			{
 				auth_warning(e,
@@ -500,6 +489,8 @@ smtp(nullserver, e)
 			define('r', protocol, e);
 			define('s', sendinghost, e);
 			initsys(e);
+			if (Errors > 0)
+				goto undo_subproc_no_pm;
 			nrcpts = 0;
 			e->e_flags |= EF_LOGSENDER|EF_CLRQUEUE;
 			setproctitle("%s %s: %.80s", e->e_id, CurSmtpClient, inp);
@@ -508,6 +499,8 @@ smtp(nullserver, e)
 			if (setjmp(TopFrame) > 0)
 			{
 				/* this failed -- undo work */
+ undo_subproc_no_pm:
+				e->e_flags &= ~EF_PM_NOTIFY;
  undo_subproc:
 				if (InChild)
 				{
@@ -525,10 +518,13 @@ smtp(nullserver, e)
 			setsender(p, e, &delimptr, ' ', FALSE);
 			if (delimptr != NULL && *delimptr != '\0')
 				*delimptr++ = '\0';
+			if (Errors > 0)
+				goto undo_subproc_no_pm;
 
 			/* do config file checking of the sender */
-			if (rscheck("check_mail", p, NULL, e) != EX_OK)
-				goto undo_subproc;
+			if (rscheck("check_mail", p, NULL, e) != EX_OK ||
+			    Errors > 0)
+				goto undo_subproc_no_pm;
 
 			/* check for possible spoofing */
 			if (RealUid != 0 && OpMode == MD_SMTP &&
@@ -579,20 +575,26 @@ smtp(nullserver, e)
 						vp == NULL ? "<null>" : vp);
 
 				mail_esmtp_args(kp, vp, e);
+				if (Errors > 0)
+					goto undo_subproc_no_pm;
 			}
+			if (Errors > 0)
+				goto undo_subproc_no_pm;
 
 			if (MaxMessageSize > 0 && e->e_msgsize > MaxMessageSize)
 			{
 				usrerr("552 Message size exceeds fixed maximum message size (%ld)",
 					MaxMessageSize);
-				/* NOTREACHED */
+				goto undo_subproc_no_pm;
 			}
 				
 			if (!enoughdiskspace(e->e_msgsize))
 			{
 				usrerr("452 Insufficient disk space; try again later");
-				break;
+				goto undo_subproc_no_pm;
 			}
+			if (Errors > 0)
+				goto undo_subproc_no_pm;
 			message("250 Sender ok");
 			gotmail = TRUE;
 			break;
@@ -615,7 +617,7 @@ smtp(nullserver, e)
 			/* limit flooding of our machine */
 			if (MaxRcptPerMsg > 0 && nrcpts >= MaxRcptPerMsg)
 			{
-				usrerr("450 Too many recipients");
+				usrerr("452 Too many recipients");
 				break;
 			}
 
@@ -626,13 +628,14 @@ smtp(nullserver, e)
 			if (p == NULL)
 				break;
 			a = parseaddr(p, NULLADDR, RF_COPYALL, ' ', &delimptr, e);
-			if (a == NULL)
+			if (a == NULL || Errors > 0)
 				break;
 			if (delimptr != NULL && *delimptr != '\0')
 				*delimptr++ = '\0';
 
 			/* do config file checking of the recipient */
-			if (rscheck("check_rcpt", p, NULL, e) != EX_OK)
+			if (rscheck("check_rcpt", p, NULL, e) != EX_OK ||
+			    Errors > 0)
 				break;
 
 			/* now parse ESMTP arguments */
@@ -673,12 +676,15 @@ smtp(nullserver, e)
 						vp == NULL ? "<null>" : vp);
 
 				rcpt_esmtp_args(a, kp, vp, e);
+				if (Errors > 0)
+					break;
 			}
+			if (Errors > 0)
+				break;
 
 			/* save in recipient list after ESMTP mods */
 			a = recipient(a, &e->e_sendqueue, 0, e);
-
-			if (Errors != 0)
+			if (Errors > 0)
 				break;
 
 			/* no errors during parsing, but might be a duplicate */
@@ -715,7 +721,8 @@ smtp(nullserver, e)
 			doublequeue = FALSE;
 			for (a = e->e_sendqueue; a != NULL; a = a->q_next)
 			{
-				if (bitset(QVERIFIED, a->q_flags))
+				if (bitset(QVERIFIED, a->q_flags) &&
+				    !bitset(EF_DISCARD, e->e_flags))
 				{
 					/* need to re-expand aliases */
 					doublequeue = TRUE;
@@ -732,9 +739,12 @@ smtp(nullserver, e)
 			SmtpPhase = "collect";
 			buffer_errors();
 			collect(InChannel, TRUE, NULL, e);
-			flush_errors(TRUE);
-			if (Errors != 0)
+			if (Errors > 0)
+			{
+				flush_errors(TRUE);
+				buffer_errors();
 				goto abortmessage;
+			}
 
 			/* make sure we actually do delivery */
 			e->e_flags &= ~EF_CLRQUEUE;
@@ -781,13 +791,19 @@ smtp(nullserver, e)
 			message("250 %s Message accepted for delivery", id);
 
 			/* if we just queued, poke it */
-			if (doublequeue && e->e_sendmode != SM_QUEUE &&
+			if (doublequeue &&
+			    e->e_sendmode != SM_QUEUE &&
 			    e->e_sendmode != SM_DEFER)
 			{
-				extern pid_t dowork();
+				CurrentLA = getla();
 
-				unlockqueue(e);
-				(void) dowork(id, TRUE, TRUE, e);
+				if (!shouldqueue(e->e_msgpriority, e->e_ctime))
+				{
+					extern pid_t dowork __P((char *, bool, bool, ENVELOPE *));
+
+					unlockqueue(e);
+					(void) dowork(id, TRUE, TRUE, e);
+				}
 			}
 
   abortmessage:
@@ -849,11 +865,16 @@ smtp(nullserver, e)
 			}
 			if (runinchild(vrfy ? "SMTP-VRFY" : "SMTP-EXPN", e) > 0)
 				break;
+			if (Errors > 0)
+				goto undo_subproc;
 			if (LogLevel > 5)
 				sm_syslog(LOG_INFO, e->e_id,
 					"%.100s: %s",
 					CurSmtpClient,
 					shortenstring(inp, 203));
+			if (setjmp(TopFrame) > 0)
+				goto undo_subproc;
+			QuickAbort = TRUE;
 			vrfyqueue = NULL;
 			if (vrfy)
 				e->e_flags |= EF_VRFYONLY;
@@ -867,12 +888,8 @@ smtp(nullserver, e)
 			{
 				(void) sendtolist(p, NULLADDR, &vrfyqueue, 0, e);
 			}
-			if (Errors != 0)
-			{
-				if (InChild)
-					finis();
-				break;
-			}
+			if (Errors > 0)
+				goto undo_subproc;
 			if (vrfyqueue == NULL)
 			{
 				usrerr("554 Nothing to %s", vrfy ? "VRFY" : "EXPN");
@@ -894,6 +911,17 @@ smtp(nullserver, e)
 			break;
 
 		  case CMDETRN:		/* etrn -- force queue flush */
+			if (bitset(PRIV_NOETRN, PrivacyFlags))
+			{
+				message("502 Sorry, we do not allow this operation");
+				if (LogLevel > 5)
+					sm_syslog(LOG_INFO, e->e_id,
+						"%.100s: %s [rejected]",
+						CurSmtpClient,
+						shortenstring(inp, 203));
+				break;
+			}
+
 			if (strlen(p) <= 0)
 			{
 				usrerr("500 Parameter required");
@@ -903,20 +931,30 @@ smtp(nullserver, e)
 			/* crude way to avoid denial-of-service attacks */
 			checksmtpattack(&n_etrn, MAXETRNCOMMANDS, "ETRN", e);
 
+			if (LogLevel > 5)
+				sm_syslog(LOG_INFO, e->e_id,
+					"%.100s: ETRN %s",
+					CurSmtpClient,
+					shortenstring(p, 203));
+
 			id = p;
 			if (*id == '@')
 				id++;
 			else
 				*--id = '@';
-			if (LogLevel > 5)
-				sm_syslog(LOG_INFO, e->e_id,
-					"%.100s: ETRN %s",
-					CurSmtpClient,
-					shortenstring(id, 203));
-			QueueLimitRecipient = id;
+				  
+			if ((new = (QUEUE_CHAR *)malloc(sizeof(QUEUE_CHAR))) == NULL)
+			{
+				syserr("500 ETRN out of memory");
+				break;
+			}
+			new->queue_match = id;
+			new->queue_next = NULL;
+			QueueLimitRecipient = new;
 			ok = runqueue(TRUE, TRUE);
+			free(QueueLimitRecipient);
 			QueueLimitRecipient = NULL;
-			if (ok)
+			if (ok && Errors == 0)
 				message("250 Queuing for node %s started", p);
 			break;
 
@@ -948,7 +986,8 @@ doquit:
 			finis();
 
 		  case CMDVERB:		/* set verbose mode */
-			if (bitset(PRIV_NOEXPN, PrivacyFlags))
+			if (bitset(PRIV_NOEXPN, PrivacyFlags) ||
+			    bitset(PRIV_NOVERB, PrivacyFlags))
 			{
 				/* this would give out the same info */
 				message("502 Verbose unavailable");
@@ -1067,7 +1106,7 @@ checksmtpattack(pcounter, maxcount, cname, e)
 
 static char *
 skipword(p, w)
-	register char *p;
+	register char *volatile p;
 	char *w;
 {
 	register char *q;
@@ -1427,12 +1466,17 @@ help(topic)
 	register FILE *hf;
 	int len;
 	bool noinfo;
+	int sff = SFF_OPENASROOT|SFF_REGONLY;
 	char buf[MAXLINE];
 	extern char Version[];
 
+	if (DontLockReadFiles)
+		sff |= SFF_NOLOCK;
+	if (!bitset(DBS_HELPFILEINUNSAFEDIRPATH, DontBlameSendmail))    
+		sff |= SFF_SAFEDIRPATH;
 
 	if (HelpFile == NULL ||
-	    (hf = safefopen(HelpFile, O_RDONLY, 0444, SFF_OPENASROOT|SFF_REGONLY|SFF_NOLOCK)) == NULL)
+	    (hf = safefopen(HelpFile, O_RDONLY, 0444, sff)) == NULL)
 	{
 		/* no help */
 		errno = 0;
